#include "gui.h"			// To avoid improper function declarations
#include "global.h"
#include "resource.h"
#include <ddraw.h>
#include <Winbase.h>
#include <d3d.h>
#include "input.h"
#include "emu.h"
#include <tchar.h>
#include "snes9x/memmap.h"
#include "bitmap.h"
#include "Direct3D.h"

#include "snes9x/snes9x.h"
#include "snes9x/cpuexec.h"

#define D3D_OVERLOADS

#define SPLASH_WAIT		1000	// Splash screen wait time, in milliseconds

#if defined MENU_640_480
	#define SCREENX			640
	#define SCREENY			480
	#define MENU_BORDER		64
	#define FONT_SIZE		32
	#define SMALL_FONT_SIZE	24
	#define ICONX			40
	#define ICONY			30
	#define LEFTOF_ICON		6
	#define LEFTOF_TEXT		8
#else
	#define SCREENX			320
	#define SCREENY			240
	#define MENU_BORDER		32
	#define FONT_SIZE		16
	#define SMALL_FONT_SIZE	12
	#define ICONX			19
	#define ICONY			15
	#define LEFTOF_ICON		3
	#define LEFTOF_TEXT		4
#endif

void DoMenu(bool allowcancel = false);
void DoSaveLoadMenu(bool);

static LPDIRECTDRAWSURFACE4 splash = NULL;



void DoTest(void) {
	LPDIRECT3DDEVICE3 dev;
	dev = GetDeviceFromSurface(gdds_back, NULL);
	D3DCOLOR c = D3DRGBA(0.5, 0.5, 0.5, 0.5);
	FillGradientRect(dev, 10, 10, 20, 20, c, c, c, c);
	FillGradientRect(dev, 20, 20, 50, 50, c, c, c, c);
	gdds_screen->Flip(NULL, DDFLIP_WAIT);
	DWORD tick = GetTickCount();							// Set the timer
	while(GetTickCount() < tick + SPLASH_WAIT);		// Wait
}








extern void DoGUI(void) {
// Added for debugging...REMOVE
//	DoTest();
//	return;
//

	JoystickInit();

#if defined USE_DIRECT_X
//#if defined USE_BITMAPS
	DWORD tick;
	DDBLTFX     ddbltfx;

	srand(GetTickCount() % 0xFFFF);

#if defined UNDER_CE
	splash = LoadBitmapFromFile("\\CD-ROM\\splash.bmp");
#else
	splash = LoadBitmapFromFile("splash.bmp");
#endif
	gdds_back->Blt(NULL, splash, NULL, DDBLT_WAIT, NULL);

	gdds_screen->Flip(NULL, DDFLIP_WAIT);

	tick = GetTickCount();							// Set the timer
	while(GetTickCount() < tick + SPLASH_WAIT);		// Wait
//#endif
#endif

#if defined DO_SNES9X
	DoSnes9xInit();
#endif

	DoMenu();
	DoSaveLoadMenu(true);

#if defined DO_SNES9X
	DoSnes9xEmulation();
#endif
}

////////////////////////////////////////////////////////////////////
// DoMenu
////////////////////////////////////////////////////////////////////
void DoMenu(bool allowcancel) {
#if defined USE_DIRECT_X
	static HFONT font = NULL;
	LOGFONT lf;
	if(!font) {
		memset(&lf, 0, sizeof(lf));
		lf.lfHeight = FONT_SIZE;
		lf.lfWeight = FW_BOLD;
		font = CreateFontIndirect(&lf);
	}

	static LPDIRECTDRAWSURFACE4 pdds = NULL;
	LPDIRECTDRAWSURFACE4 pdds2, pdds3 = NULL;
    DDSURFACEDESC2 ddsd;
	HRESULT hr;

	if(!pdds) {
		memset(&ddsd, 0, sizeof(ddsd));
		gdds_screen->GetSurfaceDesc(&ddsd);  

		ddsd.dwSize         = sizeof(DDSURFACEDESC2);
		ddsd.dwFlags        = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
		ddsd.ddsCaps.dwCaps = DDSCAPS_VIDEOMEMORY;

		ddsd.dwWidth        = SCREENX - MENU_BORDER * 2;
		ddsd.dwHeight       = SCREENY - MENU_BORDER * 2;
		hr = gpdd_directdraw->CreateSurface(&ddsd, &pdds, NULL);
		if(hr != DD_OK)
			S9xMessage(0, 0, "Failed to create menu buffer");
	}

#if defined UNDER_CE
	const char picres[3][64] = { "\\CD-ROM\\breezesintendo2.bmp",
		"\\CD-ROM\\sintendo3fade.bmp",
		"\\CD-ROM\\sintendo4.bmp" };
#else
	const char picres[3][64] = { //"breezesintendo2.bmp", "sintendo3fade.bmp", "sintendo4.bmp" };
		"sintendo4.bmp", "sintendo4.bmp", "sintendo4.bmp" };
#endif
	pdds3 = LoadBitmapFromFile(picres[rand() % 3]);
	if(pdds3 == NULL) S9xMessage(0, 0, "Error loading bitmap file");
	S9xMessage(0, 0, "Load bitmap OK");

	static LPDIRECTDRAWSURFACE4 accept = NULL, cancel = NULL, cart = NULL, folder = NULL, 
		arrow = NULL, uparrow = NULL, end = NULL;
	if(!accept) {
		accept = LoadBitmap(IDB_ACCEPT);
		cancel = LoadBitmap(IDB_CANCEL);
		cart = LoadBitmap(IDB_CART);
		folder = LoadBitmap(IDB_FOLDER);
		arrow = LoadBitmap(IDB_ARROW);
		uparrow = LoadBitmap(IDB_UPARROW);
		end = LoadBitmap(IDB_END);
	}
#endif

	int start = 0;
	int select = 0;
	int movedlast = -5;

    DDBLTFX					ddbltfx;
	DWORD					tick;

	TCHAR* filenames[128];

#if defined UNDER_CE
	HANDLE hDir;
	hDir = CreateFile(TEXT("\\CD-ROM\\SNES"),
                   GENERIC_READ,
                   FILE_SHARE_READ,
                   NULL,
                   OPEN_EXISTING,
                   FILE_FLAG_BACKUP_SEMANTICS,
                   NULL);

//	char str2[256];
//	sprintf(str2, "SNES Directory handle: %i", hDir);
//	S9xMessage(0, 0, str2);
#endif
 
	// Get file information
	WIN32_FIND_DATA fd;
	HANDLE hand;
	BOOL ok = TRUE;
	int n = 0;

#if defined UNDER_CE
	const TCHAR search[] = TEXT("\\CD-ROM\\SNES\\*.smc");
#else
	const TCHAR search[] = TEXT("SNES\\*.smc");
#endif

	hand = FindFirstFile(search, &fd);
	TCHAR* fn = new TCHAR[_tcslen(fd.cFileName) + 1];
	_tcscpy(fn, fd.cFileName);

	*(_tcschr(fn, '.')) = 0;

	filenames[n] = fn;
	n++;
	
	while(ok && n <= 128) {
		ok = FindNextFile(hand, &fd);
		if(!ok) break;
		fn = new TCHAR[_tcslen(fd.cFileName) + 1];

		_tcscpy(fn, fd.cFileName);

		*(_tcschr(fn, '.')) = 0;
		filenames[n] = fn;
		n++;
	}

#if defined USE_DIRECT_X
//#if defined USE_BITMAPS

	LPDIRECT3DVIEWPORT3 vp;
	LPDIRECT3DDEVICE3 device = GetDeviceFromSurface(gdds_back, &vp);

	memset(&ddsd, 0, sizeof(ddsd));
    gdds_screen->GetSurfaceDesc(&ddsd);  

    ddsd.dwSize         = sizeof(DDSURFACEDESC2);
    ddsd.dwFlags        = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;// | DDSCAPS_3DDEVICE;
    ddsd.ddsCaps.dwCaps = DDSCAPS_VIDEOMEMORY;

    ddsd.dwWidth        = 640;
    ddsd.dwHeight       = 480;
    hr = gpdd_directdraw->CreateSurface(&ddsd, &pdds2, NULL);
	if(hr != DD_OK) {
		char str[256];
		sprintf(str, "Error creating DirectDraw surface. %s", ErrorMsg(hr));
		S9xMessage(0, 0, str);
	}
	S9xMessage(0, 0, "DirectDraw surface created OK");
	if(pdds3 != NULL) {
		pdds2->Blt(NULL, pdds3, NULL, DDBLT_WAIT, NULL);
		pdds3->Release();
	} else {
		pdds2->Blt(NULL, splash, NULL, DDBLT_WAIT, NULL);
	}
	S9xMessage(0, 0, "Bitmap Moved");

	for(int i = 240; i >= -4;) {
		RECT srt = { 0, 240-i, 640, 240 };
		RECT drt = { 0, 0, 640, i };
		gdds_back->Blt(&drt, splash, &srt, DDBLT_WAIT, NULL);

		RECT srt2 = { 0, 240, 640, 240+i };
		RECT drt2 = { 0, 480-i, 640, 480 };
		gdds_back->Blt(&drt2, splash, &srt2, DDBLT_WAIT, NULL);

		RECT drt3 = { 0, i, 640, 480-i };
		RECT srt3 = { 0, i, 640, 480-i };
		gdds_back->Blt(&drt3, pdds2, &srt3, DDBLT_WAIT, NULL);
		gdds_screen->Flip(NULL, DDFLIP_WAIT);

/*		if(i < 32) i -= 16;
		else if(i < 64) i -= 8;
		else if(i < 192) i -= 4;
		else */ i-= 4;
	}		

	tick = GetTickCount();						// Set the timer

/*	LPDIRECT3DVIEWPORT3 vp;
	S9xMessage(0, 0, "Before GetDeviceFromSurface()");
	LPDIRECT3DDEVICE3 device = GetDeviceFromSurface(pdds2, &vp);
	S9xMessage(0, 0, "Before FillGradientRect()");
	FillGradientRect(device, MENU_BORDER, MENU_BORDER, SCREENX - MENU_BORDER, SCREENY - MENU_BORDER,
		D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5), 
		D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5));
	S9xMessage(0, 0, "After FillGradientRect()");*/
	
	gdds_back->Blt(NULL, pdds2, NULL, DDBLT_WAIT, NULL);
	gdds_screen->Flip(NULL, DDFLIP_WAIT);

	
//	S9xMessage(0, 0, "After FillGradientRect()");

//	pdds2->Release();


//#endif

	while(1) {
/*#if !defined UNDER_CE
		// Clear the back buffer
		memset(&ddbltfx, 0, sizeof(DDBLTFX));
		ddbltfx.dwSize = sizeof(DDBLTFX);
		ddbltfx.dwFillColor = 0;
		gdds_back->Blt(NULL, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);
#endif */
//		FillGradientRect(device, 0, 0, 640, 480, D3DRGBA(1,1,1,1), D3DRGBA(0.5, 0.5,0.5, 1),
	//		D3DRGBA(0.5,0.5,0.5,1), D3DRGBA(0, 0, 0, 1));
		gdds_back->Blt(NULL, pdds2, NULL, DDBLT_WAIT, NULL);

//		S9xMessage(0, 0, "Before Clear menu buffer");

		// Clear the menu buffer
/*		memset(&ddbltfx, 0, sizeof(DDBLTFX));
		ddbltfx.dwSize = sizeof(DDBLTFX);
		ddbltfx.dwFillColor = 0x01FF;
		pdds->Blt(NULL, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);

//		S9xMessage(0, 0, "After Clear menu buffer - before draw selection band");


//		S9xMessage(0, 0, "After clear selection band - Supposed to draw arrows and icons");

//		S9xMessage(0, 0, "After draw arrows and icons - Blt to back buffer"); */
// Blt to the back buffer
//		RECT rt = { MENU_BORDER, MENU_BORDER, SCREENX - MENU_BORDER, SCREENY - MENU_BORDER };
//		gdds_back->Blt(&rt, pdds, NULL, DDBLT_WAIT, NULL); 

		device->BeginScene();
//		AlphaBlt(device, pdds2, 0, 0, 640, 480, 1.0);
		FillGradientRect(device, 0,0,640,480,0xFFFFFFFF, D3DRGBA(0.5,0.5,0.5,1), D3DRGBA(0.5,0.5,0.5,1),
			D3DRGBA(0,0,0,1));

		// Draw the menu box
		FillGradientRect(device, MENU_BORDER, MENU_BORDER, SCREENX - MENU_BORDER, SCREENY - MENU_BORDER,
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5), 
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5));

		// Draw the selection band
		FillGradientRect(device, MENU_BORDER, MENU_BORDER + (select + 1) * FONT_SIZE,
			SCREENX - MENU_BORDER, MENU_BORDER + (select + 1) * FONT_SIZE + FONT_SIZE,
			D3DRGBA(0.0, 0.3, 1.0, 0.8), D3DRGBA(0.0, 0.3, 1.0, 0.8),
			D3DRGBA(0.0, 0.0, 0.3, 0.8), D3DRGBA(0.0, 0.0, 0.3, 0.8));

		device->EndScene();

		if(allowcancel) {
			int x = SCREENX/2 - 90;
			int y = SCREENY - 32 - 14;
			RECT r1 = { x, y, x + 90, y + 27 };
			gdds_back->Blt(&r1, accept, NULL, DDBLT_WAIT, NULL);
			RECT r3 = { 320, y, 320 + 90, y + 27 };
			gdds_back->Blt(&r3, cancel, NULL, DDBLT_WAIT, NULL);
		} else {
			int x = 640/2 - 46;
			int y = 480 - 32 - 14;
			RECT r1 = { x, y, x + 90, y + 27 };
			gdds_back->Blt(&r1, accept, NULL, DDBLT_WAIT, NULL);
			RECT r3 = { x+90, y, x+90+2, y + 27 };
			gdds_back->Blt(&r3, end, NULL, DDBLT_WAIT, NULL);
		}

//		S9xMessage(0, 0, "After blt to back buffer - Supposed to draw text");

		// Draw the arrows
		RECT r = { MENU_BORDER + 245, MENU_BORDER + 326,
			MENU_BORDER + 245 + 22, MENU_BORDER + 326 + 20 };
		if(n-1 >= start+10) {
			gdds_back->Blt(&r, arrow, NULL, DDBLT_WAIT, NULL);
		}
		RECT r2 = { MENU_BORDER + 245, MENU_BORDER + 6, 
			MENU_BORDER + 245 + 22, MENU_BORDER + 6 + 20 };
		if(start != 0) {
			gdds_back->Blt(&r2, uparrow, NULL, DDBLT_WAIT, NULL);
		}

		// Draw the icon
		RECT rect = { MENU_BORDER + LEFTOF_ICON, MENU_BORDER + FONT_SIZE + 1,
			MENU_BORDER + LEFTOF_ICON + ICONX, MENU_BORDER + FONT_SIZE + ICONY };
		for(int i = 0; i < 9; i++) {
			gdds_back->Blt(&rect, cart, NULL, DDBLT_WAIT, NULL);
			rect.top += FONT_SIZE;
			rect.bottom += FONT_SIZE;
		} 


		// Draw the text
		HDC	dc;
		gdds_back->GetDC(&dc);

		SelectObject(dc, font);

		SetBkMode(dc, TRANSPARENT);

		if(start == 0) {
			SIZE sz;
			SetTextColor(dc, RGB(0, 0, 0));
			GetTextExtentPoint32(dc, TEXT("Sintendo Beta v0.02"), 19, &sz);
			int x = SCREENX / 2 - sz.cx / 2;
			ExtTextOut(dc, x, MENU_BORDER, NULL, NULL, TEXT("Sintendo Beta v0.02"), 19, NULL);
		}

		SetTextColor(dc, RGB(255, 255, 255));
		int y = FONT_SIZE;
		for(int j = start; j < start + 9; j++) {
			if(filenames[j] != NULL)
			ExtTextOut(dc, MENU_BORDER + LEFTOF_ICON + ICONX + LEFTOF_TEXT, MENU_BORDER + y, NULL, NULL, filenames[j], _tcslen(filenames[j]), NULL);
			y += FONT_SIZE;
		}
		
		gdds_back->ReleaseDC(dc);

//		S9xMessage(0, 0, "Before BeginScene()");

//		device->BeginScene();
//		device->EndScene();

//		S9xMessage(0, 0, "After draw text - Blt");

	//	gdds_back->Blt(&rect, cart, NULL, DDBLT_WAIT, NULL);
		gdds_screen->Flip(NULL, DDFLIP_WAIT);

		if(!movedlast) {
			for(int d = 0; d < 4; d++) {
				int a = GetInputDeviceData(d);
				if((a | UP) == a) {
					select--;
					if(select < 0) { select = 0; start--; }
					if(start < 0) { start = 0; }
					break;
				}
				if((a | DOWN) == a) {
					select++;
					if(select > 8) { start++; select = 8; }
					if(start + 9 >= n) { start--; }
					if(select >= n) select = n - 1;
					break;
				}
				if((a | BUTTON[3]) == a) {
					select--;
					if(select < 0) { select = 0; start--; }
					if(start < 0) { start = 0; }
					break;
				}
				if((a | BUTTON[4]) == a) {
					select++;
					if(select > 8) { start++; select = 8; }
					if(start + 9 >= n) { start--; }
					if(select >= n) select = n - 1;
					break;
				}
				if((a | BUTTON[1]) == a) {
					goto end;		// ACK
				}
				if(((a | BUTTON[0]) == a) && allowcancel) {
					for(int r = 0; r < n; r++) {
						delete filenames[r];
					}
					CloseHandle(hand);
					return;
				}
				if(a) break;
			}
			movedlast++;
		} else {
			if(movedlast < 1) movedlast++;
			else movedlast = 0;
		}
	}
#endif

end:
	char str[256];
#if !defined UNDER_CE
	strcpy(str, "SNES\\");
#else
	strcpy(str, "\\CD-ROM\\SNES\\");
#endif
	char f[256];
#if defined UNDER_CE
	wcstombs(f, filenames[select+start], 255);
#else
	strcpy(f, filenames[select+start]);
#endif
	strcat(str, f);
	strcat(str, ".smc");

	Memory.LoadROM(str);

	for(int r = 0; r < n; r++) {
		delete filenames[r];
	}
	CloseHandle(hand);
}

////////////////////////////////////////////////////////////////////
// DoSaveLoadMenu
////////////////////////////////////////////////////////////////////
void DoInGameMenu(void) {
#if defined USE_DIRECT_X
	static HFONT font = NULL;
	LOGFONT lf;
	if(!font) {
		memset(&lf, 0, sizeof(lf));
		lf.lfHeight = 32;
		lf.lfWeight = FW_BOLD;
		font = CreateFontIndirect(&lf);
	}

	static LPDIRECTDRAWSURFACE4 pdds = NULL;
    DDSURFACEDESC2 ddsd;
	HRESULT hr;
	if(!pdds) {
		memset(&ddsd, 0, sizeof(ddsd));
		gdds_screen->GetSurfaceDesc(&ddsd);  

		ddsd.dwSize         = sizeof(DDSURFACEDESC2);
		ddsd.dwFlags        = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
		ddsd.ddsCaps.dwCaps = DDSCAPS_VIDEOMEMORY;

		ddsd.dwWidth        = 640 - MENU_BORDER * 2;
		ddsd.dwHeight       = 480 - MENU_BORDER * 2;
		hr = gpdd_directdraw->CreateSurface(&ddsd, &pdds, NULL);
		if(hr != DD_OK)
			S9xMessage(0, 0, "Failed to create menu buffer");
	}

	const int nOPTIONS = 6;
	TCHAR options[nOPTIONS][30] = {
		TEXT("Reset SNES"), TEXT("Select Game"), TEXT("Controller Configuration"),
		TEXT("Video Mode"), TEXT("Credits"), TEXT("Return to Game")
	};

	static LPDIRECTDRAWSURFACE4 accept = NULL, cancel = NULL;
	if(!accept) {
		accept = LoadBitmap(IDB_ACCEPT);
		cancel = LoadBitmap(IDB_CANCEL);
	}
	
	static LPDIRECTDRAWSURFACE4 optionpic[nOPTIONS] = {
		NULL, NULL, NULL, NULL, NULL, NULL };
	if(!optionpic[0]) {
		optionpic[0] = LoadBitmap(IDB_RESET);
		optionpic[1] = LoadBitmap(IDB_CART);
		optionpic[2] = LoadBitmap(IDB_CONTROLLER);
		optionpic[3] = LoadBitmap(IDB_VIDEO);
		optionpic[4] = LoadBitmap(IDB_CREDIT);
		optionpic[5] = LoadBitmap(IDB_RETURN);
	}


	int select = 0;
	int movedlast = -5;

    DDBLTFX					ddbltfx;
	DWORD					tick;

		unsigned short *sp, *bp;
		unsigned short word;
		unsigned short red, green, blue, gray;

		ddsd.dwSize = sizeof(ddsd);
		gdds_screen->Lock(NULL, &ddsd, DDLOCK_WAIT, NULL);
		sp = (unsigned short *)ddsd.lpSurface;
		gdds_back->Lock(NULL, &ddsd, DDLOCK_WAIT, NULL);
		bp = (unsigned short *)ddsd.lpSurface;

		for(int y = 0; y < 480; y++) {
			for(int x = 0; x < 640; x++) {
				word = *sp;
#if defined UNDER_CE
				red = (word & (31 << 11)) >> 11;
				green = (word & (63 << 5)) >> 6;
				blue = (word & 31);
#else
				red = (word & (31 << 10)) >> 10;
				green = (word & (31 << 5)) >> 5;
				blue = (word & 31);
#endif
				gray = 0;
				gray = (red + blue + green) / 3;

#if defined UNDER_CE
				*bp = gray << 11 + gray << 6 + gray;
#else
				*bp = (gray << 10 | gray << 5 | gray);
#endif
				sp++;
				bp++;
			}
		}

		gdds_screen->Unlock(NULL);
		gdds_back->Unlock(NULL);

//		gdds_screen->Flip(NULL, DDFLIP_WAIT);

//		S9xMessage(0, 0, "Before Clear menu buffer");

/*		int x1 = MENU_BORDER; int y1 = MENU_BORDER;
		int x2 = 640 - MENU_BORDER; int y2 = 480 - MENU_BORDER;

		gdev_device->BeginScene();
//		if(hr != D3D_OK) Beep(0, 0);
//		else Beep(0, 0);

		D3DCOLOR c = D3DRGB(0, (float)32/255, (float)180/255);

		D3DTLVERTEX vert[4];

		memset(vert, 0, sizeof(vert));

		vert[0].dvSX = (float)x1;
		vert[0].dvSY = (float)y1;
		vert[0].dvSZ = 1.0f;
		vert[0].dcColor = c;
		vert[0].rhw = 0.0f;
		vert[1].dvSX = (float)x2;
		vert[1].dvSY = (float)y1;
		vert[1].dvSZ = 1.0f;
		vert[1].dcColor = c;
		vert[1].rhw = 0.0f;
		vert[2].dvSX = (float)x1;
		vert[2].dvSY = (float)y2;
		vert[2].dvSZ = 1.0f;
		vert[2].dcColor = c;
		vert[2].rhw = 0.0f;
		vert[3].dvSX = (float)x2;
		vert[3].dvSY = (float)y2;
		vert[3].dvSZ = 1.0f;
		vert[3].dcColor = c;
		vert[3].rhw = 0.0f;

//	    gdev_device->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, TRUE );
//		gdev_device->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_BOTHSRCALPHA);

		gdev_device->DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DVT_TLVERTEX, vert, 4, D3DDP_WAIT);

		gdev_device->EndScene(); 
//		if(hr != D3D_OK) Beep(0, 0);

	// This code should not stay
    DDCAPS			ddcaps;
	HDC				dc;
	TCHAR			str[1024];
	int n;

	ddsd.dwSize = sizeof(ddsd);
	gdds_screen->GetSurfaceDesc(&ddsd);
	gdds_screen->GetDC(&dc);
	SetTextColor(dc, RGB(255, 255, 255));
	SetBkMode(dc, TRANSPARENT);

		n = wsprintf(str, TEXT("BeginScene returned %i: %s"), hr, ErrorMsg(hr));
		ExtTextOut(dc, 0, 0, NULL, NULL, str, n, NULL);

	gdds_screen->ReleaseDC(dc);

*/
	while(1) {

		// Clear the menu buffer
		memset(&ddbltfx, 0, sizeof(DDBLTFX));
		ddbltfx.dwSize = sizeof(DDBLTFX);
		ddbltfx.dwFillColor = 0x01FF;
		pdds->Blt(NULL, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);


//		S9xMessage(0, 0, "After Clear menu buffer - before draw selection band");

		// Draw the selection band
		memset(&ddbltfx, 0, sizeof(DDBLTFX));
		ddbltfx.dwSize = sizeof(DDBLTFX);
		ddbltfx.dwFillColor = 20 + (8 << 5);
		RECT rct;
		rct.left = 0;
		rct.right = 640 - MENU_BORDER *2;
//		select++;
		rct.top = (select + 1) * 32;
		rct.bottom = rct.top + 32;
		pdds->Blt(&rct, NULL, &rct, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);

//		S9xMessage(0, 0, "After clear selection band - Supposed to draw arrows and icons");

		// Draw the icon
		RECT rect = { LEFTOF_ICON, FONT_SIZE+1, LEFTOF_ICON + ICONX, FONT_SIZE + ICONY };
		for(int i = 0; i < 9; i++) {
			pdds->Blt(&rect, optionpic[i], NULL, DDBLT_WAIT, NULL);
			rect.top += FONT_SIZE;
			rect.bottom += FONT_SIZE;
		} 

//		S9xMessage(0, 0, "After draw arrows and icons - Blt to back buffer"); */
// Blt to the back buffer
		RECT rt = { MENU_BORDER, MENU_BORDER, 640 - MENU_BORDER, 480 - MENU_BORDER };
		gdds_back->Blt(&rt, pdds, NULL, DDBLT_WAIT, NULL); 

		int x = 640/2 - 90;
		int y = 480 - 32 - 14;
		RECT r1 = { x, y, x + 90, y + 27 };
		gdds_back->Blt(&r1, accept, NULL, DDBLT_WAIT, NULL);
		RECT r3 = { 320, y, 320 + 90, y + 27 };
		gdds_back->Blt(&r3, cancel, NULL, DDBLT_WAIT, NULL);


//		S9xMessage(0, 0, "After blt to back buffer - Supposed to draw text");

		// Draw the text
		HDC	dc;
		gdds_back->GetDC(&dc);

		SelectObject(dc, font);

		SetBkMode(dc, TRANSPARENT);

		SetTextColor(dc, RGB(0, 0, 0));
		ExtTextOut(dc, 195, 64, NULL, NULL, TEXT("Game Paused"), 11, NULL);

		SetTextColor(dc, RGB(255, 255, 255));
		y = 32;
		for(int j = 0; j < nOPTIONS; j++) {
			if(options[j] != NULL)
			ExtTextOut(dc, 64 + 5 + 38 + 8, 64 + y, NULL, NULL, options[j], _tcslen(options[j]), NULL);
			y += 32;
		}
		
		gdds_back->ReleaseDC(dc); 

//		S9xMessage(0, 0, "After draw text - Blt");*/

//		gdds_back->Blt(&rect, cart, NULL, DDBLT_WAIT, NULL); 
		gdds_screen->Flip(NULL, DDFLIP_WAIT);

		if(!movedlast) {
			for(int d = 0; d < 4; d++) {
				int a = GetInputDeviceData(d);
				if((a | UP) == a) {
					select--;
					if(select < 0) { select = 0; }
					break;
				}
				if((a | DOWN) == a) {
					select++;
					if(select >= nOPTIONS) { select = nOPTIONS - 1; }
					break;
				}
				if((a | BUTTON[3]) == a) {
					select--;
					if(select < 0) { select = 0; }
					break;
				}
				if((a | BUTTON[4]) == a) {
					select++;
					if(select >= nOPTIONS) { select = nOPTIONS - 1; }
					break;
				}
				if((a | BUTTON[1]) == a) {
					switch(select) {
					case 0:
						S9xReset();
						return;
					case 1:
						DoMenu(true);
						return;
					case 2:
					case 3:
					case 4:
					case 5:
						return;
					}
					return;
				}
				if((a | BUTTON[0]) == a) {
					return;
				}
				if(a) break;
			}
			movedlast++;
		} else {
			if(movedlast < 1) movedlast++;
			else movedlast = 0;
		}

		gdds_back->Blt(NULL, gdds_screen, NULL, DDBLT_WAIT, NULL);
	}

#endif
}

////////////////////////////////////////////////////////////////////
// DoSaveLoadMenu
////////////////////////////////////////////////////////////////////
void DoSaveLoadMenu(bool save) {
#ifdef DO_SAVING
	bool available[3][4];
	memset(available, 0, sizeof(bool) * 12);

	static HFONT font = NULL, smallfont = NULL;
	LOGFONT lf;
	if(!font) {
		memset(&lf, 0, sizeof(lf));
		lf.lfHeight = 32;
		lf.lfWeight = FW_BOLD;
		font = CreateFontIndirect(&lf);

		memset(&lf, 0, sizeof(lf));
		lf.lfHeight = 24;
		//lf.lfWeight = FW_BOLD;
		smallfont = CreateFontIndirect(&lf);
	}

	static LPDIRECTDRAWSURFACE4 pdds = NULL;
    DDSURFACEDESC2 ddsd;
	HRESULT hr;
	if(!pdds) {
		memset(&ddsd, 0, sizeof(ddsd));
		gdds_screen->GetSurfaceDesc(&ddsd);  

		ddsd.dwSize         = sizeof(DDSURFACEDESC2);
		ddsd.dwFlags        = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
		ddsd.ddsCaps.dwCaps = DDSCAPS_VIDEOMEMORY;

		ddsd.dwWidth        = 640 - MENU_BORDER * 2;
		ddsd.dwHeight       = 480 - MENU_BORDER * 2;
		hr = gpdd_directdraw->CreateSurface(&ddsd, &pdds, NULL);
		if(hr != DD_OK)
			S9xMessage(0, 0, "Failed to create menu buffer");
	}

	static LPDIRECTDRAWSURFACE4 accept = NULL, cancel = NULL;
	if(!accept) {
		accept = LoadBitmap(IDB_ACCEPT);
		cancel = LoadBitmap(IDB_CANCEL);
	}
	
	int select = 0;
	int movedlast = -5;

    DDBLTFX					ddbltfx;
	DWORD					tick;


//	while(1) {

		// Clear the menu buffer
		memset(&ddbltfx, 0, sizeof(DDBLTFX));
		ddbltfx.dwSize = sizeof(DDBLTFX);
		ddbltfx.dwFillColor = 0xFFFF;
		pdds->Blt(NULL, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);

		RECT rct = { 1, 1, 640 - MENU_BORDER * 2 - 1, 480 - MENU_BORDER * 2 - 1 };
		memset(&ddbltfx, 0, sizeof(DDBLTFX));
		ddbltfx.dwSize = sizeof(DDBLTFX);
		ddbltfx.dwFillColor = 0x01FF;
		pdds->Blt(&rct, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);
		// Draw all the boxes

		RECT rect = { 16, 46, 128-16, 70 };
		for(int i = 0; i < 4; i++) {
			memset(&ddbltfx, 0, sizeof(DDBLTFX));
			ddbltfx.dwSize = sizeof(DDBLTFX);
			ddbltfx.dwFillColor = 0xFFFF;
			pdds->Blt(&rect, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);
			rect.top++;
			rect.bottom--;
			rect.left++;
			rect.right--;

			memset(&ddbltfx, 0, sizeof(DDBLTFX));
			ddbltfx.dwSize = sizeof(DDBLTFX);
			if(available[0][i]) 
				ddbltfx.dwFillColor = 17 << 11 | 40 << 5 | 31;
			else
				ddbltfx.dwFillColor = 16 << 11 | 32 << 5 | 16;
			pdds->Blt(&rect, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);
			rect.top--;
			rect.bottom++;
			rect.left--;
			rect.right++;
			rect.left += 128;
			rect.right += 128;
		}

		RECT rect2 = { 16, 70 + 32, 128-16, 70 + 128 };
		for(int j = 0; j < 2; j++) {
			for(int i = 0; i < 4; i++) {
				memset(&ddbltfx, 0, sizeof(DDBLTFX));
				ddbltfx.dwSize = sizeof(DDBLTFX);
				ddbltfx.dwFillColor = 0xFFFF;
				pdds->Blt(&rect2, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);
				rect.top++;
				rect.bottom--;
				rect.left++;
				rect.right--;

				memset(&ddbltfx, 0, sizeof(DDBLTFX));
				ddbltfx.dwSize = sizeof(DDBLTFX);
				ddbltfx.dwFillColor = 17 << 11 | 40 << 5 | 31;
				pdds->Blt(&rect2, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);
				rect.top--;
				rect.bottom++;
				rect.left--;
				rect.right++;
				rect2.left += 128;
				rect2.right += 128;
			}
			rect2.left = 16;
			rect2.right = 128-16;
			rect2.top += 128-16;
			rect2.bottom += 128-16;
		}

		
		// Draw the text
		HDC	dc;
		pdds->GetDC(&dc);

		SelectObject(dc, font);
		SetBkMode(dc, TRANSPARENT);

		SetTextColor(dc, RGB(0, 0, 0));
		if(save) {
			ExtTextOut(dc, 180, 0, NULL, NULL, TEXT("Save to:"), 8, NULL);
		} else {
			ExtTextOut(dc, 175, 0, NULL, NULL, TEXT("Load from:"), 10, NULL);
		}

		SelectObject(dc, smallfont);
		SetTextColor(dc, RGB(255, 255, 255));
		int x = 38;
		for(j = 0; j < 4; j++) {
			if(available[0][i]) 
				SetTextColor(dc, RGB(255, 255, 255));
			else
				SetTextColor(dc, RGB(64, 64, 64));
			TCHAR str[256];
			wsprintf(str, TEXT("File %i"), j);
			ExtTextOut(dc, x, 46, NULL, NULL, str, _tcslen(str), NULL);
			x += 128;
		}
		
		pdds->ReleaseDC(dc); 

		
		/*
		// Draw the selection band
		memset(&ddbltfx, 0, sizeof(DDBLTFX));
		ddbltfx.dwSize = sizeof(DDBLTFX);
		ddbltfx.dwFillColor = 20 + (8 << 5);
		RECT rct;
		rct.left = 0;
		rct.right = 640 - MENU_BORDER *2;
//		select++;
		rct.top = (select + 1) * 32;
		rct.bottom = rct.top + 32;
		pdds->Blt(&rct, NULL, &rct, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);

//		S9xMessage(0, 0, "After clear selection band - Supposed to draw arrows and icons");

		// Draw the icon
		RECT rect = { 5, 33, 42, 62 };
		for(int i = 0; i < nOPTIONS; i++) {
			pdds->Blt(&rect, optionpic[i], NULL, DDBLT_WAIT, NULL);
			rect.top += 32;
			rect.bottom += 32;
		} 

//		S9xMessage(0, 0, "After draw arrows and icons - Blt to back buffer"); */
// Blt to the back buffer
		RECT rt = { MENU_BORDER, MENU_BORDER, 640 - MENU_BORDER, 480 - MENU_BORDER };
		gdds_back->Blt(&rt, pdds, NULL, DDBLT_WAIT, NULL); 

/*		int x = 640/2 - 90;
		int y = 480 - 32 - 14;
		RECT r1 = { x, y, x + 90, y + 27 };
		gdds_back->Blt(&r1, accept, NULL, DDBLT_WAIT, NULL);
		RECT r3 = { 320, y, 320 + 90, y + 27 };
		gdds_back->Blt(&r3, cancel, NULL, DDBLT_WAIT, NULL);


//		S9xMessage(0, 0, "After blt to back buffer - Supposed to draw text");


//		S9xMessage(0, 0, "After draw text - Blt");*/

//		gdds_back->Blt(&rect, cart, NULL, DDBLT_WAIT, NULL); 
		gdds_screen->Flip(NULL, DDFLIP_WAIT);

/*		if(!movedlast) {
			for(int d = 0; d < 4; d++) {
				int a = GetInputDeviceData(d);
				if((a | UP) == a) {
					select--;
					if(select < 0) { select = 0; }
					break;
				}
				if((a | DOWN) == a) {
					select++;
					if(select >= nOPTIONS) { select = nOPTIONS - 1; }
					break;
				}
				if((a | BUTTON[3]) == a) {
					select--;
					if(select < 0) { select = 0; }
					break;
				}
				if((a | BUTTON[4]) == a) {
					select++;
					if(select >= nOPTIONS) { select = nOPTIONS - 1; }
					break;
				}
				if((a | BUTTON[1]) == a) {
					switch(select) {
					case 0:
						S9xReset();
						return;
					case 1:
						DoMenu(true);
						return;
					case 2:
					case 3:
					case 4:
					case 5:
						return;
					}
					return;
				}
				if((a | BUTTON[0]) == a) {
					return;
				}
				if(a) break;
			}
			movedlast++;
		} else {
			if(movedlast < 1) movedlast++;
			else movedlast = 0;
		}

		gdds_back->Blt(NULL, gdds_screen, NULL, DDBLT_WAIT, NULL);
	} */
//	}

	MSG msg;
	while(1) {
		if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE)) {
			if(!GetMessage(&msg, NULL, 0, 0))
				return;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
#endif

}

